@@ -4,9 +4,11 @@
 
 
 # Modules
-import numpy as np
-from scipy.special import binom, factorial
 import itertools
+import warnings
+import numpy as np
+import matplotlib.pyplot as plt
+from scipy.special import binom, factorial
 from tqdm import tqdm
 
 class ShapleyLorenzShare:
@@ -29,6 +31,9 @@ class ShapleyLorenzShare:
         self.y_bg = y_background
         self.N = self.data.data.shape[0]
         self.M = self.data.data.shape[1]
+
+        if self.N > 50:
+            warnings.warn('a large background dataset may cause prohibitive long runtime')
         
         #Dimension of null_model
         null_model = self.model.f(self.data.data)
@@ -120,6 +125,24 @@ class ShapleyLorenzShare:
     
         return (1/(2*mu_pol*(n**2)))*Gin_pol
 
+    # Plotting tool
+    def slz_plots(self, tuple = False):
+        '''
+        Creates a plot of the lorenz zonoid values
+        
+        Parameter:
+        tuple : boolean (Default = False)
+            specifies, whether to plot a seperate graph for the
+            class tuple, in case of multiple classes
+        '''
+        if tuple == False:
+            plt.bar(lz_shares[:,0], lz_shares[:,1], color = (0.0,0.36,0.8,0.8))
+            plt.title('Shapley Lorenz Zonoid Values', fontweight = 'bold', fontsize = '14')
+            plt.ylabel('shapley LZ value')
+            plt.xlabel('features')
+            plt.show()
+
+    # Shapley Lorenz Zonoid function
     def shapleyLorenz_val(self, X, y, class_prob = False, pred_out = 'predict', **kwargs):
         '''
         Computes the Shapley Lorenz marginal contribution for
@@ -159,19 +182,20 @@ class ShapleyLorenzShare:
         assert X.shape[0] == len(y), 'Covariate matrix and response vector need to have the same number of observations'
 
         #Initiate variables globally
-        self.N_test = X.shape[0]
+        X = convert_to_data(X) # standardise data
+        self.N_test = X.data.shape[0]
         self.row = kwargs.get('row', None)
         if self.row == False:
             assert isinstance(self.row, (int, list, np.ndarray)), "not a valid row type. Needs to be either 'int', 'list', or 'array'"
 
-        if class_prob == True:
+        if class_prob == True and (pred_out == 'predict_proba' or pred_out == 'predict_log_proba'):
             val, num = np.unique(self.y_bg, return_counts = True)
             if len(val) == 1:
                 raise ValueError('only observations from one class included in provided data. Need to have at least one observation from each class')
             self.y_class_mu = np.zeros((val.shape[0],1))
             for i in range(val.shape[0]):
                 self.y_class_mu[i] = num[i]/self.N
-        elif class_prob == False:
+        elif class_prob == False or (class_prob == True and pred_out == 'predict'):
             self.y_mu = np.mean(self.y_bg)
 
         #Container for output
@@ -188,15 +212,15 @@ class ShapleyLorenzShare:
             V_k = np.zeros((self.N, self.M, 2**(self.M-1)))
             kernel = np.zeros((2**(self.M-1),1))
             if class_prob == False or (class_prob == True and pred_out == 'predict'):
-                y_base = np.zeros((self.N, 2**(self.M-1)))
-                y_k = np.zeros((self.N, 2**(self.M-1)))
+                y_base = np.zeros((self.N_test, 2**(self.M-1)))
+                y_k = np.zeros((self.N_test, 2**(self.M-1)))
             elif class_prob == True and pred_out == 'predict_proba':
-                y_base = np.zeros((self.N, 2, 2**(self.M-1)))
-                y_b0 = np.zeros((self.N, 2**(self.M-1)))
-                y_b1 = np.zeros((self.N, 2**(self.M-1)))
-                y_k = np.zeros((self.N, 2, 2**(self.M-1)))
-                y_k0 = np.zeros((self.N, 2**(self.M-1)))
-                y_k1 = np.zeros((self.N, 2**(self.M-1)))
+                y_base = np.zeros((self.N_test, 2, 2**(self.M-1)))
+                y_b0 = np.zeros((self.N_test, 2**(self.M-1)))
+                y_b1 = np.zeros((self.N_test, 2**(self.M-1)))
+                y_k = np.zeros((self.N_test, 2, 2**(self.M-1)))
+                y_k0 = np.zeros((self.N_test, 2**(self.M-1)))
+                y_k1 = np.zeros((self.N_test, 2**(self.M-1)))
             
             #Initialise indexes
             s_all = list(range(self.M))
@@ -219,6 +243,13 @@ class ShapleyLorenzShare:
                 #for single row
                 if self.row == False:
                     if type(self.row) == int:
+                        if len(s) == 0:
+                                if i != 0:
+                                    raise ValueError('s is empty for i not equal 0')
+                                continue
+                        else:
+                            V_base[:,s,i] = X.data[n_test, s]
+                        V_k[:,s_k,i] = X.data[n_test, s_k]
                         if class_prob == False or (class_prob == True and pred_out == 'predict'):
                             y_k[:,i] = self.model.f(V_k[self.row,:,i])
 
@@ -244,8 +275,8 @@ class ShapleyLorenzShare:
                                     raise ValueError('s is empty for i not equal 0')
                                 continue
                             else:
-                                V_base[:,s,i] = X[n_test, s]
-                            V_k[:,s_k,i] = X[n_test,s_k]
+                                V_base[:,s,i] = X.data[n_test, s]
+                            V_k[:,s_k,i] = X.data[n_test,s_k]
 
                             if self.row == False and isinstance(self.row, (list, np.ndarray)):
                                 if class_prob == False or (class_prob == True and pred_out == 'predict'):
@@ -268,14 +299,14 @@ class ShapleyLorenzShare:
                                         ("Not a valid method. Valid methods are: 'predict', 'predict_proba' and 'predict_log_proba'")
 
                 # No specified rows
-                for n_test in range(X.shape[0]):
+                for n_test in range(self.N_test):
                     if len(s) == 0:
                         if i != 0:
                             raise ValueError('s is empty for i not equal 0')
                         continue
                     else:
-                        V_base[:,s,i] = X[n_test, s]
-                    V_k[:,s_k,i] = X[n_test,s_k]
+                        V_base[:,s,i] = X.data[n_test, s]
+                    V_k[:,s_k,i] = X.data[n_test,s_k]
                     #print('\nV_base initial shape: {}'.format(V_base.shape))
 
                     #Compute predicted values with model w and w/o kth covariat, if no row(s) specified
@@ -315,31 +346,33 @@ class ShapleyLorenzShare:
                         y_b1[:,i] = np.sort(y_base[:,1,i],0)
 
             #Compute Lorenz Zenoid values
-            Lor_val_temp = np.zeros((self.N,2**(self.M-1)))
-            Lor_val_temp0 = np.zeros((self.N,2**(self.M-1)))
-            Lor_val_temp1 = np.zeros((self.N,2**(self.M-1)))
+            Lor_val_temp = np.zeros((self.N_test,2**(self.M-1)))
+            Lor_val_temp0 = np.zeros((self.N_test,2**(self.M-1)))
+            Lor_val_temp1 = np.zeros((self.N_test,2**(self.M-1)))
 
             if class_prob == False or (class_prob == True and pred_out == 'predict'):
-                for j in range(self.N):
+                for j in range(self.N_test):
                     Lor_val_temp[j,:] = j*(y_k[j,:]-y_base[j,:]) # for all feature combinations simultaneously
                 Lor_val_temp_sum = np.sum(Lor_val_temp,0)
                 #Lor_val_pol = self.lz_polarisation(Lor_val_temp,self.M) # polarisation in case of negative values
 
-                Lor_val = ((2/(self.N**2))*self.y_mu)*Lor_val_temp_sum
+                Lor_val = ((2/(self.N_test**2))*self.y_mu)*Lor_val_temp_sum
                 Lor_val = Lor_val.reshape((1,2**(self.M-1)))
 
                 LZ[k,0] = np.dot(Lor_val,kernel) # equation 19 on page 10 of Giudiuci and Raffinetti (Feb 2020) paper
 
             elif class_prob == True and (pred_out == 'predict_proba' or pred_out == 'predict_log_proba'):
-                for j in range(self.N):
+                for j in range(self.N_test):
                     Lor_val_temp0[j,:] = j*(y_k0[j,:]-y_b0[j,:])
                     Lor_val_temp1[j,:] = j*(y_k1[j,:]-y_b1[j,:])
+                Lor_val_temp0_sum = np.sum(Lor_val_temp0,0)
+                Lor_val_temp1_sum = np.sum(Lor_val_temp1,0)
                 
                 #Lor_val0_pol = self.lz_polarisation(Lor_val_temp0,self.M)
                 #Lor_val1_pol = self.lz_polarisation(Lor_val_temp1,self.M)
 
-                Lor_val0 = ((2/(self.N**2))*self.y_class_mu[0])*Lor_val_temp0
-                Lor_val1 = ((2/(self.N**2))*self.y_class_mu[1])*Lor_val_temp1
+                Lor_val0 = ((2/(self.N_test**2))*self.y_class_mu[0])*Lor_val_temp0_sum
+                Lor_val1 = ((2/(self.N_test**2))*self.y_class_mu[1])*Lor_val_temp1_sum
                 Lor_val0 = Lor_val0.reshape((1,2**(self.M-1)))
                 Lor_val1 = Lor_val1.reshape((1,2**(self.M-1)))
 
@@ -347,10 +380,10 @@ class ShapleyLorenzShare:
                 LZ1[k,0] = np.dot(Lor_val1,kernel)
 
         if class_prob == False or (class_prob == True and pred_out == 'predict'):
-            return LZ
+            return np.column_stack((X.col_names,LZ))
 
         elif class_prob == True and (pred_out == 'predict_proba' or pred_out == 'predict_log_proba'):
-            return LZ0, LZ1;
+            return np.column_stack((X.col_names,LZ0)), np.column_stack((X.col_names, LZ1));
 
 
 #Auxiliary functions
@@ -385,44 +418,4 @@ def convert_to_model(value):
     if isinstance(value, Model):
         return value
     else:
-        return Model(value)
-
-
-def powerset(iterable):
-        '''
-        Creates index vectors of length 0-M of 'iterable'
-        '''
-        s = list(iterable)
-        return itertools.chain.from_iterable(itertools.combinations(s,r)\
-            for r in range(len(s)+1))
-            #s : iterable
-            #r : length
-
-#---------------------------------
-#Tests
-# from sklearn.ensemble import RandomForestRegressor
-# from sklearn.linear_model import LinearRegression
-# from sklearn.metrics import mean_squared_error as mse
-
-# # Sim data
-# # background data
-# X_bg = np.abs(np.random.normal(0,1,(100,16)))
-# betas = np.random.uniform(1,6,16)
-# y_bg = np.dot(X_bg,betas)
-
-# # data to explain
-# X = np.abs(np.random.normal(0,1,(3,16)))
-# #X_onerow = np.abs(np.random.normal(0,1,(2,16)))
-# y = np.dot(X, betas)
-
-# model = LinearRegression()
-# #rf_model = RandomForestRegressor()
-# model.fit(X_bg,y_bg)
-# #rf_model.fit(X_bg, y_bg)
-# yhat = model.predict(X)
-# meSE = mse(y,yhat)
-
-# lorenzshare = ShapleyLorenzShare(model.predict, X_bg, y_bg)
-# lz_shares = lorenzshare.shapleyLorenz_val(X, y)
-# lz_shares.tolist()
-# betas.tolist()
+        return Model(value)
\ No newline at end of file
